@@ -9,5 +9,6 @@ classes
 from .dynesty import NestedSampler, DynamicNestedSampler
 from . import bounding
 from . import utils
+from . import pool
 
-__version__ = "1.2.3"
+from ._version import __version__
