@@ -191,8 +191,7 @@ def _get_update_interval_ratio(update_interval, sample, bound, ndim, nlive,
             update_interval_frac = np.inf
             warnings.warn(
                 "No update_interval set with unknown sampling method: "
-                f"'{sample}'. Defaulting to no updates."
-            )
+                f"'{sample}'. Defaulting to no updates.")
     elif isinstance(update_interval, float):
         update_interval_frac = update_interval
     elif isinstance(update_interval, int):
@@ -331,36 +330,51 @@ def stopping_function(results,
         return stop <= 1.
 
 
-def initialize_live_points(live_points,
-                           prior_transform,
-                           loglikelihood,
-                           M,
-                           nlive=None,
-                           npdim=None,
-                           rstate=None,
-                           blob=False,
-                           use_pool_ptform=None):
+def _initialize_live_points(live_points,
+                            prior_transform,
+                            loglikelihood,
+                            M,
+                            nlive=None,
+                            npdim=None,
+                            rstate=None,
+                            blob=False,
+                            use_pool_ptform=None):
     """
     Initialize the first set of live points before starting the sampling
 
-    Parameters:
+    Parameters
+    ----------
     live_points: tuple of arrays or None
-        This can be either none or tuple of 3 arrays (u, v, logl), i.e.
-        point location in cube coordinates, point location in original
-        coordinates, and logl values
+        This can be either none or
+        tuple of 3 arrays (u, v, logl) or
+        tuple of 4 arrays (u, v, logl, blobs), i.e.
+        points location in cube coordinates,
+        point slocation in original coordinates,
+        logl values and optionally blobs associated
+
     prior_transform: function
+
     log_likelihood: function
+
     M: function
         The function supporting parallel calls like M(func, list)
+
     nlive: int
         Number of live-points
+
     npdim: int
         Number of dimensions
+
     rstate: :class: numpy.random.RandomGenerator
+
+    blob: bool
+        If true we also keep track of blobs returned by likelihood
+
     use_pool_ptform: bool or None
         The flag to perform prior transform using multiprocessing pool or not
 
-    Returns:
+    Returns
+    -------
     (live_u, live_v, live_logl, blobs): tuple
         The tuple of arrays.
         The first is in unit cube coordinates.
@@ -373,7 +387,7 @@ def initialize_live_points(live_points,
         # sampling from the unit cube.
         n_attempts = 100
         for _ in range(n_attempts):
-            live_u = rstate.uniform(size=(nlive, npdim))
+            live_u = rstate.random(size=(nlive, npdim))
             if use_pool_ptform:
                 live_v = M(prior_transform, np.asarray(live_u))
             else:
@@ -412,8 +426,10 @@ def initialize_live_points(live_points,
     else:
         # If live points were provided, convert the log-likelihoods and
         # then run a quick safety check.
-        live_u, live_v, live_logl, blobs = live_points
-
+        live_u, live_v, live_logl = live_points[:3]
+        if blob:
+            live_blobs = live_points[3]
+        live_logl = np.asarray(live_logl)
         for i, logl in enumerate(live_logl):
             if not np.isfinite(logl):
                 if np.sign(logl) < 0:
@@ -423,7 +439,7 @@ def initialize_live_points(live_points,
                                      "point {1} located at u={2} v={3} "
                                      " is invalid.".format(
                                          logl, i, live_u[i], live_v[i]))
-        if all(live_logl == _LOWL_VAL):
+        if np.all(live_logl == _LOWL_VAL):
             raise ValueError("Not a single provided live point has a "
                              "valid log-likelihood!")
     if (np.ptp(live_logl) == 0):
@@ -573,6 +589,8 @@ class DynamicSampler:
         self.nlive_init = None
         self.batch_sampler = None
 
+        self.live_blobs = None
+
     def __setstate__(self, state):
         self.__dict__ = state
         self.pool = None
@@ -779,7 +797,9 @@ class DynamicSampler:
         n_effective: int, optional
             This option is deprecated and will be removed in a future release.
 
-        live_points : list of 3 `~numpy.ndarray` each with shape (nlive, ndim)
+        live_points: list of 3 `~numpy.ndarray` each with shape (nlive, ndim)
+            and optionally list of blobs associated with these likelihood calls
+            (if blob=True in the sampler)
             A set of live points used to initialize the nested sampling run.
             Contains `live_u`, the coordinates on the unit cube, `live_v`, the
             transformed variables, and `live_logl`, the associated
@@ -865,7 +885,7 @@ class DynamicSampler:
             self.reset()
 
             (self.live_u, self.live_v, self.live_logl,
-             blobs) = initialize_live_points(
+             blobs) = _initialize_live_points(
                  live_points,
                  self.prior_transform,
                  self.loglikelihood,
@@ -1161,16 +1181,17 @@ class DynamicSampler:
             if psel:
                 # If the lower bound encompasses all saved samples, we want
                 # to propose a new set of points from the unit cube.
-                live_u, live_v, live_logl, live_blobs = initialize_live_points(
-                    None,
-                    self.prior_transform,
-                    self.loglikelihood,
-                    self.M,
-                    nlive=nlive_new,
-                    npdim=self.npdim,
-                    rstate=self.rstate,
-                    blob=self.blob,
-                    use_pool_ptform=self.use_pool_ptform)
+                (live_u, live_v, live_logl,
+                 live_blobs) = _initialize_live_points(
+                     None,
+                     self.prior_transform,
+                     self.loglikelihood,
+                     self.M,
+                     nlive=nlive_new,
+                     npdim=self.npdim,
+                     rstate=self.rstate,
+                     blob=self.blob,
+                     use_pool_ptform=self.use_pool_ptform)
 
                 live_bound = np.zeros(nlive_new, dtype=int)
                 live_it = np.zeros(nlive_new, dtype=int) + self.it
@@ -1732,7 +1753,9 @@ class DynamicSampler:
             A function that prints out the current state of the sampler.
             If not provided, the default :meth:`results.print_fn` is used.
 
-        live_points : list of 3 `~numpy.ndarray` each with shape (nlive, ndim)
+        live_points: list of 3 `~numpy.ndarray` each with shape (nlive, ndim)
+            and optionally list of blobs associated with these likelihood calls
+            (if blob=True in the sampler)
             A set of live points used to initialize the nested sampling run.
             Contains `live_u`, the coordinates on the unit cube, `live_v`, the
             transformed variables, and `live_logl`, the associated
@@ -1740,7 +1763,8 @@ class DynamicSampler:
             set of live points will be drawn from the unit `npdim`-cube.
             **WARNING: It is crucial that the initial set of live points have
             been sampled from the prior. Failure to provide a set of valid
-            live points will result in biased results.**
+            live points will lead to incorrect results.**
+
         resume: bool, optional
             If resume is set to true, we will try to resume a previously
             interrupted run
@@ -1893,6 +1917,10 @@ class DynamicSampler:
                 else:
                     # We didn't run a single batch but now we're done!
                     break
+            if checkpoint_file is not None:
+                # In the very end I save the checkpoint no matter
+                # the timing
+                self.save(checkpoint_file)
         finally:
             if pbar is not None:
                 pbar.close()
@@ -1985,7 +2013,8 @@ class DynamicSampler:
             file every checkpoint_every seconds
         checkpoint_every: float, optional
             The number of seconds between checkpoints that will save
-            the internal state of the sampler
+            the internal state of the sampler. The sampler will also be
+            saved in the end of the run irrespective of checkpoint_every.
         """
 
         # Initialize values.
