@@ -89,9 +89,9 @@ class Sampler:
         self.ncdim = ncdim
         self.blob = blob
         # live points
-        self.live_u, self.live_v, self.live_logl, blobs = live_points
+        self.live_u, self.live_v, self.live_logl = live_points[:3]
         if blob:
-            self.live_blobs = blobs
+            self.live_blobs = live_points[3]
         else:
             self.live_blobs = None
         self.nlive = len(self.live_u)
@@ -180,7 +180,7 @@ class Sampler:
         """Re-initialize the sampler."""
 
         # live points
-        self.live_u = self.rstate.uniform(size=(self.nlive, self.npdim))
+        self.live_u = self.rstate.random(size=(self.nlive, self.npdim))
         if self.use_pool_ptform:
             # Use the pool to compute the prior transform.
             self.live_v = np.array(
@@ -328,8 +328,8 @@ class Sampler:
                 self.nqueue += 1
         else:
             # Propose/evaluate points directly from the unit cube.
-            point_queue = self.rstate.uniform(size=(self.queue_size -
-                                                    self.nqueue, self.npdim))
+            point_queue = self.rstate.random(size=(self.queue_size -
+                                                   self.nqueue, self.npdim))
             axes_queue = np.identity(
                 self.ncdim)[None, :, :] + np.zeros(self.queue_size -
                                                    self.nqueue)[:, None, None]
@@ -913,7 +913,8 @@ class Sampler:
 
         checkpoint_every: float, optional
             The number of seconds between checkpoints that will save
-            the internal state of the sampler
+            the internal state of the sampler. The sampler will also be
+            saved in the end of the run irrespective of checkpoint_every.
         """
 
         # Check for deprecated options
