@@ -8,8 +8,8 @@ Functions for proposing new live points used by
 
 """
 
+from collections import namedtuple
 import warnings
-import math
 import numpy as np
 from numpy import linalg
 
@@ -21,8 +21,10 @@ __all__ = [
     "sample_hslice"
 ]
 
-EPS = float(np.finfo(np.float64).eps)
-SQRTEPS = math.sqrt(float(np.finfo(np.float64).eps))
+SamplerArgument = namedtuple('SamplerArgument', [
+    'u', 'loglstar', 'axes', 'scale', 'prior_transform', 'loglikelihood',
+    'rseed', 'kwargs'
+])
 
 
 def sample_unif(args):
@@ -80,16 +82,14 @@ def sample_unif(args):
     """
 
     # Unzipping.
-    (u, loglstar, axes, scale, prior_transform, loglikelihood, rseed,
-     kwargs) = args
 
     # Evaluate.
-    v = prior_transform(np.asarray(u))
-    logl = loglikelihood(np.asarray(v))
+    v = args.prior_transform(np.asarray(args.u))
+    logl = args.loglikelihood(np.asarray(v))
     nc = 1
     blob = None
 
-    return u, v, logl, nc, blob
+    return args.u, v, logl, nc, blob
 
 
 def sample_rwalk(args):
@@ -145,11 +145,10 @@ def sample_rwalk(args):
     """
 
     # Unzipping.
-    (u, loglstar, axes, scale, prior_transform, loglikelihood, rseed,
-     kwargs) = args
-    rstate = get_random_generator(rseed)
-    return generic_random_walk(u, loglstar, axes, scale, prior_transform,
-                               loglikelihood, rstate, kwargs)
+    rstate = get_random_generator(args.rseed)
+    return generic_random_walk(args.u, args.loglstar, args.axes, args.scale,
+                               args.prior_transform, args.loglikelihood,
+                               rstate, args.kwargs)
 
 
 def generic_random_walk(u, loglstar, axes, scale, prior_transform,
@@ -290,7 +289,7 @@ def propose_ball_point(u,
 
     # draw random point for non clustering parameters
     # we only need to generate them once
-    u_non_cluster = rstate.uniform(0, 1, n - n_cluster)
+    u_non_cluster = rstate.random(n - n_cluster)
     u_prop = np.zeros(n)
     u_prop[n_cluster:] = u_non_cluster
 
@@ -317,8 +316,45 @@ def propose_ball_point(u,
         return None, True
 
 
+def _slice_doubling_accept(x1, F, loglstar, L, R, fL, fR):
+    """
+    Acceptance test of slice sampling when doubling mode is used.
+    This is an exact implementation of algorithm 6 of Neal 2003
+    here w=1 and x0=0 as we are working in the
+    coordinate system of F(A) = f(x0+A*w)
+
+    Arguments are
+    1) candidate location x1
+    2) wrapped logl function (see generic_slice_step)
+    3) threshold logl value
+    4) left edge of the full interval
+    5) right edge of the full interval
+    6) value at left edge
+    7) value at right edge
+    """
+    lhat, rhat = L, R
+    f_lhat = fL
+    f_rhat = fR
+    D = False
+    while rhat - lhat > 1.1:
+        # Define slice and window.
+        M = (lhat + rhat) / 2.
+        # Propose new position.
+        if (0 < M <= x1) or (x1 < M <= 0):
+            D = True
+        if x1 < M:
+            rhat = M
+            f_rhat = F(rhat)[1]
+        else:
+            lhat = M
+            f_lhat = F(lhat)[1]
+        if D and loglstar >= f_lhat and loglstar >= f_rhat:
+            return False
+    return True
+
+
 def generic_slice_step(u, direction, nonperiodic, loglstar, loglikelihood,
-                       prior_transform, rstate):
+                       prior_transform, doubling, rstate):
     """
     Do a slice generic slice sampling step along a specified dimension
 
@@ -337,9 +373,9 @@ def generic_slice_step(u, direction, nonperiodic, loglstar, loglikelihood,
     rstate: random state
     """
     nc, nexpand, ncontract = 0, 0, 0
-    nexpand_threshold = 10000  # Threshold for warning the user
+    nexpand_threshold = 1000  # Threshold for warning the user
     n = len(u)
-    rand0 = rstate.uniform()  # initial scale/offset
+    rand0 = rstate.random()  # initial scale/offset
     dirlen = linalg.norm(direction)
     maxlen = np.sqrt(n) / 2.
     # maximum initial interval length (the diagonal of the cube)
@@ -368,21 +404,40 @@ def generic_slice_step(u, direction, nonperiodic, loglstar, loglikelihood,
 
     logl_l = F(nstep_l)[1]
     logl_r = F(nstep_r)[1]
+    expansion_warning = False
+    if not doubling:
+        # "Stepping out" the left and right bounds.
+        while logl_l > loglstar:
+            nstep_l -= 1
+            logl_l = F(nstep_l)[1]
+            nexpand += 1
+        while logl_r > loglstar:
+            nstep_r += 1
+            logl_r = F(nstep_r)[1]
+            nexpand += 1
+        if nexpand > nexpand_threshold:
+            expansion_warning = True
+            warnings.warn('The slice sample interval was expanded more '
+                          f'than {nexpand_threshold} times')
+
+    else:
+        # "Stepping out" the left and right bounds.
+        K = 1
+        while (logl_l > loglstar or logl_r > loglstar):
+            V = rstate.random()
+            if V < 0.5:
+                nstep_l -= (nstep_r - nstep_l)
+                logl_l = F(nstep_l)[1]
+            else:
+                nstep_r += (nstep_r - nstep_l)
+                logl_r = F(nstep_r)[1]
+            nexpand += K
+            K *= 2
+        L = nstep_l
+        R = nstep_r
+        fL = logl_l
+        fR = logl_r
 
-    # "Stepping out" the left and right bounds.
-    while logl_l > loglstar:
-        nstep_l -= 1
-        logl_l = F(nstep_l)[1]
-        nexpand += 1
-    while logl_r > loglstar:
-        nstep_r += 1
-        logl_r = F(nstep_r)[1]
-        nexpand += 1
-    if nexpand > nexpand_threshold:
-        warnings.warn(
-            str.format(
-                'The slice sample interval was expanded more than {0} times',
-                nexpand_threshold))
     # Sample within limits. If the sample is not valid, shrink
     # the limits until we hit the `loglstar` bound.
 
@@ -391,13 +446,15 @@ def generic_slice_step(u, direction, nonperiodic, loglstar, loglikelihood,
         nstep_hat = nstep_r - nstep_l
 
         # Propose new position.
-        nstep_prop = nstep_l + rstate.uniform() * nstep_hat  # scale from left
+        nstep_prop = nstep_l + rstate.random() * nstep_hat  # scale from left
         u_prop, logl_prop = F(nstep_prop)
         ncontract += 1
 
         # If we succeed, move to the new position.
-        if logl_prop > loglstar:
-            fscale = (nstep_r - nstep_l) / dirnorm
+        # note that if we are using doubling mode we accept only
+        # if _slice_doubling_accept() returns True
+        if logl_prop > loglstar and (not doubling or _slice_doubling_accept(
+                nstep_prop, F, loglstar, L, R, fL, fR)):
             break
         # If we fail, check if the new point is to the left/right of
         # our original point along our proposal axis and update
@@ -412,18 +469,16 @@ def generic_slice_step(u, direction, nonperiodic, loglstar, loglikelihood,
                 raise RuntimeError("Slice sampler has failed to find "
                                    "a valid point. Some useful "
                                    "output quantities:\n"
-                                   "u: {0}\n"
-                                   "nstep_left: {1}\n"
-                                   "nstep_right: {2}\n"
-                                   "nstep_hat: {3}\n"
-                                   "u_prop: {4}\n"
-                                   "loglstar: {5}\n"
-                                   "logl_prop: {6}\n"
-                                   "direction: {7}\n".format(
-                                       u, nstep_l, nstep_r, nstep_hat, u_prop,
-                                       loglstar, logl_prop, direction))
+                                   f"u: {u}\n"
+                                   f"nstep_left: {nstep_l}\n"
+                                   f"nstep_right: {nstep_r}\n"
+                                   f"nstep_hat: {nstep_hat}\n"
+                                   f"u_prop: {u_prop}\n"
+                                   f"loglstar: {loglstar}\n"
+                                   f"logl_prop: {logl_prop}\n"
+                                   f"direction: {direction}\n")
     v_prop = prior_transform(u_prop)
-    return u_prop, v_prop, logl_prop, nc, nexpand, ncontract, fscale
+    return u_prop, v_prop, logl_prop, nc, nexpand, ncontract, expansion_warning
 
 
 def sample_slice(args):
@@ -480,12 +535,13 @@ def sample_slice(args):
     """
 
     # Unzipping.
-    (u, loglstar, axes, scale, prior_transform, loglikelihood, rseed,
-     kwargs) = args
-    rstate = get_random_generator(rseed)
+    (u, loglstar, axes, scale, prior_transform, loglikelihood,
+     kwargs) = (args.u, args.loglstar, args.axes, args.scale,
+                args.prior_transform, args.loglikelihood, args.kwargs)
+    rstate = get_random_generator(args.rseed)
     # Periodicity.
     nonperiodic = kwargs.get('nonperiodic', None)
-
+    doubling = kwargs.get('slice_doubling', False)
     # Setup.
     n = len(u)
     assert axes.shape[0] == n
@@ -493,13 +549,12 @@ def sample_slice(args):
     nc = 0
     nexpand = 0
     ncontract = 0
-    fscale = []
 
     # Modifying axes and computing lengths.
     axes = scale * axes.T  # scale based on past tuning
-
+    expansion_warning_set = False
     # Slice sampling loop.
-    for it in range(slices):
+    for _ in range(slices):
 
         # Shuffle axis update order.
         idxs = np.arange(n)
@@ -511,19 +566,25 @@ def sample_slice(args):
             # Select axis.
             axis = axes[idx]
             (u_prop, v_prop, logl_prop, nc1, nexpand1, ncontract1,
-             fscale1) = generic_slice_step(u, axis, nonperiodic, loglstar,
-                                           loglikelihood, prior_transform,
-                                           rstate)
+             expansion_warning) = generic_slice_step(u, axis, nonperiodic,
+                                                     loglstar, loglikelihood,
+                                                     prior_transform, doubling,
+                                                     rstate)
             u = u_prop
             nc += nc1
             nexpand += nexpand1
             ncontract += ncontract1
-            fscale.append(fscale1)
-
+            if expansion_warning and not doubling:
+                # if we expanded the interval by more than
+                # the threshold we set the warning and enable doubling
+                expansion_warning_set = True
+                doubling = True
+                warnings.warn('Enabling doubling strategy of slice '
+                              'sampling from Neal(2003)')
     blob = {
-        'fscale': np.mean(fscale),
         'nexpand': nexpand,
-        'ncontract': ncontract
+        'ncontract': ncontract,
+        'expansion_warning_set': expansion_warning_set
     }
 
     return u_prop, v_prop, logl_prop, nc, blob
@@ -581,11 +642,13 @@ def sample_rslice(args):
     """
 
     # Unzipping.
-    (u, loglstar, axes, scale, prior_transform, loglikelihood, rseed,
-     kwargs) = args
-    rstate = get_random_generator(rseed)
+    (u, loglstar, axes, scale, prior_transform, loglikelihood,
+     kwargs) = (args.u, args.loglstar, args.axes, args.scale,
+                args.prior_transform, args.loglikelihood, args.kwargs)
+    rstate = get_random_generator(args.rseed)
     # Periodicity.
     nonperiodic = kwargs.get('nonperiodic', None)
+    doubling = kwargs.get('slice_doubling', False)
 
     # Setup.
     n = len(u)
@@ -594,10 +657,10 @@ def sample_rslice(args):
     nc = 0
     nexpand = 0
     ncontract = 0
-    fscale = []
+    expansion_warning_set = False
 
     # Slice sampling loop.
-    for it in range(slices):
+    for _ in range(slices):
 
         # Propose a direction on the unit n-sphere.
         drhat = rstate.standard_normal(size=n)
@@ -607,18 +670,24 @@ def sample_rslice(args):
         direction = np.dot(axes, drhat) * scale
 
         (u_prop, v_prop, logl_prop, nc1, nexpand1, ncontract1,
-         fscale1) = generic_slice_step(u, direction, nonperiodic, loglstar,
-                                       loglikelihood, prior_transform, rstate)
+         expansion_warning) = generic_slice_step(u, direction, nonperiodic,
+                                                 loglstar, loglikelihood,
+                                                 prior_transform, doubling,
+                                                 rstate)
         u = u_prop
         nc += nc1
         nexpand += nexpand1
         ncontract += ncontract1
-        fscale.append(fscale1)
+        if expansion_warning and not doubling:
+            doubling = True
+            expansion_warning_set = True
+            warnings.warn('Enabling doubling strategy of slice '
+                          'sampling from Neal(2003)')
 
     blob = {
-        'fscale': np.mean(fscale),
         'nexpand': nexpand,
-        'ncontract': ncontract
+        'ncontract': ncontract,
+        'expansion_warning_set': expansion_warning_set
     }
 
     return u_prop, v_prop, logl_prop, nc, blob
@@ -680,9 +749,11 @@ def sample_hslice(args):
     """
 
     # Unzipping.
-    (u, loglstar, axes, scale, prior_transform, loglikelihood, rseed,
-     kwargs) = args
-    rstate = get_random_generator(rseed)
+    (u, loglstar, axes, scale, prior_transform, loglikelihood,
+     kwargs) = (args.u, args.loglstar, args.axes, args.scale,
+                args.prior_transform, args.loglikelihood, args.kwargs)
+    rstate = get_random_generator(args.rseed)
+
     # Periodicity.
     nonperiodic = kwargs.get('nonperiodic', None)
 
@@ -700,7 +771,7 @@ def sample_hslice(args):
     ncontract = 0
 
     # Slice sampling loop.
-    for it in range(slices):
+    for _ in range(slices):
         # Define the left, "inner", and right "nodes" for a given chord.
         # We will plan to slice sampling using these chords.
         nodes_l, nodes_m, nodes_r = [], [], []
@@ -758,7 +829,7 @@ def sample_hslice(args):
                         logl_out = logl_r
                     # Check if we could compute gradients assuming we
                     # terminated with the current `u_out`.
-                    if np.isfinite(logl_out):
+                    if np.isfinite(float(logl_out)):
                         reverse = False
                     else:
                         reverse = True
@@ -794,7 +865,7 @@ def sample_hslice(args):
                     u_r_r[i] += 1e-10
                     if unitcheck(u_r_r, nonperiodic):
                         v_r_r = prior_transform(np.asarray(u_r_r))
-                        logl_r_r = loglikelihood(np.asarray(v_r_r))
+                        logl_r_r = loglikelihood(np.asarray(v_r_r)).val
                     else:
                         logl_r_r = -np.inf
                         reverse = True  # can't compute gradient
@@ -803,7 +874,7 @@ def sample_hslice(args):
                     u_r_l[i] -= 1e-10
                     if unitcheck(u_r_l, nonperiodic):
                         v_r_l = prior_transform(np.asarray(u_r_l))
-                        logl_r_l = loglikelihood(np.asarray(v_r_l))
+                        logl_r_l = loglikelihood(np.asarray(v_r_l)).val
                     else:
                         logl_r_l = -np.inf
                         reverse = True  # can't compute gradient
@@ -897,7 +968,7 @@ def sample_hslice(args):
                         logl_out = logl_l
                     # Check if we could compute gradients assuming we
                     # terminated with the current `u_out`.
-                    if np.isfinite(logl_out):
+                    if np.isfinite(float(logl_out)):
                         reverse = False
                     else:
                         reverse = True
@@ -933,7 +1004,7 @@ def sample_hslice(args):
                     u_l_r[i] += 1e-10
                     if unitcheck(u_l_r, nonperiodic):
                         v_l_r = prior_transform(np.asarray(u_l_r))
-                        logl_l_r = loglikelihood(np.asarray(v_l_r))
+                        logl_l_r = loglikelihood(np.asarray(v_l_r)).val
                     else:
                         logl_l_r = -np.inf
                         reverse = True  # can't compute gradient
@@ -942,7 +1013,7 @@ def sample_hslice(args):
                     u_l_l[i] -= 1e-10
                     if unitcheck(u_l_l, nonperiodic):
                         v_l_l = prior_transform(np.asarray(u_l_l))
-                        logl_l_l = loglikelihood(np.asarray(v_l_l))
+                        logl_l_l = loglikelihood(np.asarray(v_l_l)).val
                     else:
                         logl_l_l = -np.inf
                         reverse = True  # can't compute gradient
@@ -1007,7 +1078,7 @@ def sample_hslice(args):
         nodes_l, nodes_m, nodes_r = (np.array(nodes_l), np.array(nodes_m),
                                      np.array(nodes_r))
         Nchords = len(nodes_l)
-        axlen = np.zeros(Nchords, dtype='float')
+        axlen = np.zeros(Nchords, dtype=float)
         for i, (nl, nr) in enumerate(zip(nodes_l, nodes_r)):
             axlen[i] = linalg.norm(nr - nl)
 
@@ -1019,11 +1090,8 @@ def sample_hslice(args):
             if np.any(axlen < 1e-5 * axlen_init):
                 raise RuntimeError("Hamiltonian slice sampling appears to be "
                                    "stuck! Some useful output quantities:\n"
-                                   "u: {0}\n"
-                                   "u_left: {1}\n"
-                                   "u_right: {2}\n"
-                                   "loglstar: {3}.".format(
-                                       u, u_l, u_r, loglstar))
+                                   f"u: {u}\n u_left: {u_l}\n"
+                                   f"u_right: {u_r}\n loglstar: {loglstar}.")
 
             # Select chord.
             axprob = axlen / np.sum(axlen)
@@ -1031,7 +1099,7 @@ def sample_hslice(args):
             # Define chord.
             u_l, u_m, u_r = nodes_l[idx], nodes_m[idx], nodes_r[idx]
             u_hat = u_r - u_l
-            rprop = rstate.uniform()
+            rprop = rstate.random()
             u_prop = u_l + rprop * u_hat  # scale from left
             if unitcheck(u_prop, nonperiodic):
                 v_prop = prior_transform(np.asarray(u_prop))
@@ -1056,18 +1124,14 @@ def sample_hslice(args):
                     nodes_r[idx] = u_prop
                     axlen[idx] *= rprop
                 else:
-                    raise RuntimeError("Slice sampler has failed to find "
-                                       "a valid point. Some useful "
-                                       "output quantities:\n"
-                                       "u: {0}\n"
-                                       "u_left: {1}\n"
-                                       "u_right: {2}\n"
-                                       "u_hat: {3}\n"
-                                       "u_prop: {4}\n"
-                                       "loglstar: {5}\n"
-                                       "logl_prop: {6}.".format(
-                                           u, u_l, u_r, u_hat, u_prop,
-                                           loglstar, logl_prop))
+                    raise RuntimeError(
+                        "Slice sampler has failed to find "
+                        "a valid point. Some useful "
+                        "output quantities:\n"
+                        f"u: {u}\n u_left: {u_l}\n"
+                        f"u_right: {u_r}\n u_hat: {u_hat}\n"
+                        f"u_prop: {u_prop}\n loglstar: {loglstar}\n"
+                        f"logl_prop: {logl_prop}")
 
     blob = {'nmove': nmove, 'nreflect': nreflect, 'ncontract': ncontract}
 
