@@ -725,7 +725,7 @@ Return the list of items in the results object as list of key,value pairs
 
     def samples_equal(self, rstate=None):
         """
-        Return the equally weighted samples
+        Return the equally weighted samples in random order.
         """
         if rstate is None:
             rstate = get_random_generator()
@@ -815,7 +815,8 @@ def get_enlarge_bootstrap(sample, enlarge, bootstrap):
 def get_nonbounded(ndim, periodic, reflective):
     """
     Return a boolean mask for dimensions that are either
-    periodic or reflective
+    periodic or reflective. It will be true for normal dimension
+    and false for periodic/reflective
     """
     if periodic is not None and reflective is not None:
         if np.intersect1d(periodic, reflective) != 0:
@@ -825,12 +826,12 @@ def get_nonbounded(ndim, periodic, reflective):
     if periodic is not None or reflective is not None:
         nonbounded = np.ones(ndim, dtype=bool)
         if periodic is not None:
-            if np.max(periodic) > ndim:
+            if np.max(periodic) >= ndim:
                 raise ValueError(
                     'Incorrect periodic variable index (larger than ndim')
             nonbounded[periodic] = False
         if reflective is not None:
-            if np.max(reflective) > ndim:
+            if np.max(reflective) >= ndim:
                 raise ValueError(
                     'Incorrect periodic variable index (larger than ndim')
             nonbounded[reflective] = False
@@ -980,8 +981,9 @@ def mean_and_cov(samples, weights):
 
 def resample_equal(samples, weights, rstate=None):
     """
-    Resample a new set of points from the weighted set of inputs
-    such that they all have equal weight.
+    Resample a set of points from the weighted set of inputs
+    such that they all have equal weight. The points are also
+    randomly shuffled.
 
     Each input sample appears in the output array either
     `floor(weights[i] * nsamples)` or `ceil(weights[i] * nsamples)` times,
@@ -1001,7 +1003,7 @@ def resample_equal(samples, weights, rstate=None):
     Returns
     -------
     equal_weight_samples : `~numpy.ndarray` with shape (nsamples,)
-        New set of samples with equal weights.
+        New set of samples with equal weights in random order.
 
     Examples
     --------
@@ -1044,7 +1046,7 @@ def resample_equal(samples, weights, rstate=None):
         else:
             j += 1
 
-    return samples[idx]
+    return rstate.permutation(samples[idx])
 
 
 def quantile(x, q, weights=None):
